import MazeGenerator
import Settings
import random
from mathutils import Vector

__author__ = 'Kevin'


def GenerateLevel(cont):

    spawn_controller = cont.owner
    wall_object = spawn_controller.scene.objectsInactive[spawn_controller["Wall"]]
    start_object = spawn_controller.scene.objectsInactive[spawn_controller["Start"]]
    end_object = spawn_controller.scene.objectsInactive[spawn_controller["End"]]
    grass_object = spawn_controller.scene.objectsInactive[spawn_controller["Grass"]]
    tourist_object = spawn_controller.scene.objectsInactive[spawn_controller["Tourist"]]

    difficulty = Settings.Difficulty
    maze_size = Settings.GetMazeGeneratorSettingsForDifficulty(difficulty)
    seed = 928793048 #random.randint(0, 1000000000)

    generator = MazeGenerator.MazeGenerator(maze_size["Width"], maze_size["Height"], seed, 'false', 0)
    generator.GenerateMaze()

    n_tiles = len(generator.Tiles)
    for i in range(len(generator.Tiles)):

        if generator.Tiles[i] == "0":
            row = generator.GetCurrentRow(i) - generator.Height/2
            col = generator.GetCurrentColumn(i) - generator.Width/2
            if random.randint(0, 25) == 0:
                spawn_controller.scene.addObject(grass_object, spawn_controller).worldPosition = Vector([int(col), int(row), 0.04])

        elif generator.Tiles[i] == "1" and NextToFloorTile(generator, i):
            row = generator.GetCurrentRow(i) - generator.Height/2
            col = generator.GetCurrentColumn(i) - generator.Width/2
            spawn_controller.scene.addObject(wall_object, spawn_controller).worldPosition = Vector([int(col), int(row), 0.5])

        elif generator.Tiles[i] == "2":
            row = generator.GetCurrentRow(i) - generator.Height/2
            col = generator.GetCurrentColumn(i) - generator.Width/2
            spawn_controller.scene.addObject(start_object, spawn_controller).worldPosition = Vector([int(col), int(row), 0.5])

        elif generator.Tiles[i] == "3":
            row = generator.GetCurrentRow(i) - generator.Height/2
            col = generator.GetCurrentColumn(i) - generator.Width/2
            spawn_controller.scene.addObject(end_object, spawn_controller).worldPosition = Vector([int(col), int(row), 0.5])

        if i % 1000 == 0:
            progress = "\rGenerated: %s/%s (%s)" % (i, n_tiles, i/n_tiles*100)
            print(progress)

    floor_object = spawn_controller.scene.objectsInactive[spawn_controller["Floor"]]
    phy_floor_object = spawn_controller.scene.addObject(floor_object, spawn_controller)

    phy_floor_object.worldPosition = Vector([0, 0, 0])
    phy_floor_object.worldScale = Vector([generator.Height+1, generator.Width+1, 1])

    CreateBoarder(generator, spawn_controller, wall_object)
    AddTourists(generator, spawn_controller, tourist_object)


def NextToFloorTile(generator, tile_index):

    floor = "0"
    start = "2"
    end = "3"
    tiles = generator.Tiles.copy()

    new_position = generator.TransformUp(tile_index)
    if new_position != tile_index:
        if tiles[new_position] == floor or tiles[new_position] == start or tiles[new_position] == end:
            return True

    new_position = generator.TransformRight(tile_index)
    if new_position != tile_index:
        if tiles[new_position] == floor or tiles[new_position] == start or tiles[new_position] == end:
            return True

    new_position = generator.TransformDown(tile_index)
    if new_position != tile_index:
        if tiles[new_position] == floor or tiles[new_position] == start or tiles[new_position] == end:
            return True

    new_position = generator.TransformLeft(tile_index)
    if new_position != tile_index:
        if tiles[new_position] == floor or tiles[new_position] == start or tiles[new_position] == end:
            return True

    return False


def CreateBoarder(generator, spawn_controller, wall_object):

    y = generator.Height / 2-0.5
    x = 0
    z = 0.5
    wall = spawn_controller.scene.addObject(wall_object, spawn_controller)
    wall.worldPosition = Vector([x, y, z])
    wall.worldScale = Vector([generator.Width+1, 1, 1])

    y = generator.Height / 2 - generator.Height - 0.5
    x = 0
    wall = spawn_controller.scene.addObject(wall_object, spawn_controller)
    wall.worldPosition = Vector([x, y, z])
    wall.worldScale = Vector([generator.Width+1, 1, 1])

    y = 0
    x = generator.Width/2-0.5
    wall = spawn_controller.scene.addObject(wall_object, spawn_controller)
    wall.worldPosition = Vector([x, y, z])
    wall.worldScale = Vector([1, generator.Height+1, 1])

    y = 0
    x = generator.Width/2 - generator.Width - 0.5
    wall = spawn_controller.scene.addObject(wall_object, spawn_controller)
    wall.worldPosition = Vector([x, y, z])
    wall.worldScale = Vector([1, generator.Height+1, 1])


def AddTourists(generator, spawn_controller, tourist_object):

    number_tourists_to_spawn = Settings.Tourist_NumberOfTourists
    floor_count = 0

    for i in range(len(generator.Tiles)):

        if generator.Tiles[i] == "0":
            floor_count += 1

    tourist_spacing = int(floor_count / number_tourists_to_spawn)

    for i in range(len(generator.Tiles)):

        if generator.Tiles[i] == "0" and i % tourist_spacing == 0:
            row = generator.GetCurrentRow(i) - generator.Height/2
            col = generator.GetCurrentColumn(i) - generator.Width/2
            phy_tourist_object = spawn_controller.scene.addObject(tourist_object, spawn_controller)
            phy_tourist_object.worldPosition = Vector([col, row, 1])